package cn.iocoder.yudao.module.wealth.controller.admin.receiptItem;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.util.collectors.CollectorsUtil;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.idempotent.core.annotation.Idempotent;
import cn.iocoder.yudao.module.ecw.api.currency.CurrencyApi;
import cn.iocoder.yudao.module.ecw.api.currency.dto.CurrencyRespDTO;
import cn.iocoder.yudao.module.system.dal.dataobject.user.AdminUserDO;
import cn.iocoder.yudao.module.system.service.user.AdminUserService;
import cn.iocoder.yudao.module.wealth.convert.receiptItem.ReceiptItemConvert;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receiptAccount.ReceiptAccountDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receiptApproval.ReceiptApprovalDO;
import cn.iocoder.yudao.module.wealth.dal.dataobject.receiptItem.ReceiptItemDO;
import cn.iocoder.yudao.module.wealth.dal.mysql.receiptAccount.ReceiptAccountMapper;
import cn.iocoder.yudao.module.wealth.dal.mysql.receiptApproval.ReceiptApprovalMapper;
import cn.iocoder.yudao.module.wealth.dal.mysql.receiptItem.ReceiptItemMapper;
import cn.iocoder.yudao.module.wealth.service.receiptItem.ReceiptItemService;
import cn.iocoder.yudao.module.wealth.vo.receipt.FinanceReceiptItemCancelVo;
import cn.iocoder.yudao.module.wealth.vo.receipt.FinanceReceiptItemVo;
import cn.iocoder.yudao.module.wealth.vo.receiptAccount.ReceiptItemBatchCreateReqVO;
import cn.iocoder.yudao.module.wealth.vo.receiptAccount.ReceiptItemBatchRespVO;
import cn.iocoder.yudao.module.wealth.vo.receiptItem.ReceiptItemBackVO;
import cn.iocoder.yudao.module.wealth.vo.receiptItem.ReceiptItemCreateReqVO;
import cn.iocoder.yudao.module.wealth.vo.receiptItem.ReceiptItemUpdateReqVO;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.http.MediaType;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Validated
@RestController
@Api(tags = "管理后台 - 收款明细")
@RequestMapping("/ecw/receipt-item")
public class ReceiptItemController {

    @Resource
    private ReceiptItemService receiptItemService;

    @Resource
    private ReceiptAccountMapper receiptAccountMapper;

    @Resource
    private ReceiptItemMapper receiptItemMapper;

    @Resource
    private CurrencyApi currencyApi;

    @Resource
    private AdminUserService adminUserService;

    @Resource
    private ReceiptApprovalMapper receiptApprovalMapper;

    @PostMapping("/create")
    @ApiOperation("创建收款明细")
    //@PreAuthorize("@ss.hasPermission('ecw:receipt-item:create')")
    public CommonResult<Long> createReceiptItem(@Valid @RequestBody ReceiptItemCreateReqVO createReqVO) {
        return success(receiptItemService.createReceiptItem(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新收款明细")
    //@PreAuthorize("@ss.hasPermission('ecw:receipt-item:update')")
    public CommonResult<Boolean> updateReceiptItem(@Valid @RequestBody ReceiptItemUpdateReqVO updateReqVO) {
        receiptItemService.updateReceiptItem(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除收款明细")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    //@PreAuthorize("@ss.hasPermission('ecw:receipt-item:delete')")
    public CommonResult<Boolean> deleteReceiptItem(@RequestParam("id") Long id) {
        receiptItemService.deleteReceiptItem(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得收款明细详情")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<ReceiptItemBackVO> getReceiptItem(@RequestParam("id") Long id) {
        ReceiptItemDO receiptItem = receiptItemService.getReceiptItem(id);
        ReceiptItemBackVO v = ReceiptItemConvert.INSTANCE.convert(receiptItem);
        //计算剩余金额
        //BigDecimal p = receiptItemService.getWriteOffAmountByReceiptId(id);
        //BigDecimal t = receiptAccountMapper.getWriteOffAmountByReceiptId(id);
        getSY(v, v.getReceiptId());
        SetData(v);
        return success(v);
    }

    private void SetData(ReceiptItemBackVO v) {
        List<AdminUserDO> ListUser = adminUserService.getUsersList();
        Map<Long, String> userMap =
                ListUser.stream().collect(
                        Collectors.toMap((AdminUserDO::getId),
                                AdminUserDO::getNickname));
        String sT = "";
        if (v.getCreator() != null) {
            sT = userMap.get(Long.parseLong(v.getCreator()));
            v.setCreator(sT);
        }
        if (v.getUpdater() != null && v.getUpdater().length() > 0) {
            sT = userMap.get(Long.parseLong(v.getUpdater()));
            v.setUpdater(sT);
        }

        if (v.getBmpId() == null||v.getBmpId().length()==0) {
            v.setCreateBpm("");
            v.setCreateTimeBpm(null);
            v.setUpdateTimeBpm(null);
            return;
        }

        LambdaQueryWrapper<ReceiptApprovalDO> lambdaQueryWrapper =
                new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(ReceiptApprovalDO::getBmpId, v.getBmpId());
        List<ReceiptApprovalDO> list =
                receiptApprovalMapper.selectList(lambdaQueryWrapper);
        if (list==null||list.size() == 0) return;

        //审批创建人
        if (list.get(0).getCreator() != null
                && list.get(0).getCreator().length() > 0) {
            sT = userMap.get(Long.parseLong(list.get(0).getCreator()));
            v.setCreateBpm(sT);

            //审批创建日期
            if (list.get(0).getCreateTime() != null)
                v.setCreateTimeBpm(list.get(0).getCreateTime());

            //审批通过日期
            if (list.get(0).getUpdateTime() != null)
                v.setUpdateTimeBpm(list.get(0).getUpdateTime());
        }
    }


    /*
     获取剩余金额
     */
    private BigDecimal getSY(ReceiptItemBackVO v, long id) {
        Map<Integer, CurrencyRespDTO> currencyMap = currencyApi.getAllCurrency();

        BigDecimal bResult = new BigDecimal(0);
        LambdaQueryWrapper<ReceiptAccountDO> lambdaQueryWrapper2 =
                new LambdaQueryWrapper<>();
        lambdaQueryWrapper2.eq(ReceiptAccountDO::getReceiptId, id);
        List<ReceiptAccountDO> ReceiptAccountList =
                receiptAccountMapper.selectList(lambdaQueryWrapper2);

        BigDecimal t = new BigDecimal(0);
        Long currencyId = null;
        if (CollectionUtil.isEmpty(ReceiptAccountList) == true) {
            t = new BigDecimal(0);
        } else {
            //获取币种
            currencyId = ReceiptAccountList.get(0).getCurrencyId();
            t = ReceiptAccountList.stream().
                    collect(CollectorsUtil.summingBigDecimal(
                            ReceiptAccountDO::getWriteOffAmount));
        }

        LambdaQueryWrapper<ReceiptItemDO> lambdaQueryWrapper =
                new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(ReceiptItemDO::getReceiptId, id);
        List<ReceiptItemDO> listItem =
                receiptItemMapper.selectList(lambdaQueryWrapper);

        BigDecimal p = new BigDecimal(0);
        if (CollectionUtil.isEmpty(listItem) == true) {
            p = new BigDecimal(0);
        } else {
            //对指定列求和
            p = listItem.stream().
                    collect(CollectorsUtil.summingBigDecimal(
                            ReceiptItemDO::getWriteOffAmount));
        }
        bResult = t.subtract(p);
        String bzName = "";
        if (currencyId != null) {
            int iId = Integer.parseInt(currencyId.toString());
            bzName = currencyMap.get(iId).getTitleZh();
        }
        v.setSyValue(bResult.toString() + bzName);
        return bResult;
    }


    @GetMapping("/getReceivableItem")
    @ApiOperation("根据收款单ID获得收款明细列表")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    public CommonResult<List<ReceiptItemBackVO>> getUpdateReceivableItemDetail(@RequestParam("id") Long id) {
        LambdaQueryWrapper<ReceiptItemDO> lambdaQueryWrapper = new LambdaQueryWrapper();
        lambdaQueryWrapper.eq(ReceiptItemDO::getReceiptId, id);
        List<ReceiptItemDO> list = receiptItemService.list(lambdaQueryWrapper);
        return success(ReceiptItemConvert.INSTANCE.convertList(list));
    }

    @PostMapping("/financeReceiptItemWriteOff")
    @ApiOperation("收款明细核销")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> createFinanceReceiptItemWriteOff(@RequestBody FinanceReceiptItemVo financeReceiptVo) {
        receiptItemService.createFinanceReceiptItemWriteOff(financeReceiptVo);
        return success(true);
    }

    /*
     批量提交审核功能
     {
     "receiptId": "12",
     "receiptNo": "SKD00000011",
     "receiptItemId": "4"
     }
     */
    @PostMapping("/financeReceiptItemWriteOff2")
    @ApiOperation("收款明细核销")
    @Idempotent(timeout = 15)
    public CommonResult<Boolean> createFinanceReceiptItemWriteOff2(
            @RequestBody List<FinanceReceiptItemVo> financeReceiptVoList) {
        if (CollectionUtil.isEmpty(financeReceiptVoList)) {
            return success(true);
        }
        for (FinanceReceiptItemVo financeReceiptVo : financeReceiptVoList
        ) {
            receiptItemService.createFinanceReceiptItemWriteOff(financeReceiptVo);
        }

        return success(true);
    }

    @PostMapping("/cancelFinanceReceiptItemWriteOff")
    @ApiOperation("取消收款明细核销")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> cancelFinanceReceiptItemWriteOff(@RequestBody FinanceReceiptItemCancelVo
                                                                          financeReceiptVo) {
        receiptItemService.cancelFinanceReceiptItemWriteOff(financeReceiptVo);
        return success(true);
    }

    /*
     收款单反核销
     */
    @PostMapping("/financeReceiptItemWriteOffNo")
    @ApiOperation("收款明细反核销")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> createFinanceReceiptItemWriteOffNo(
            @RequestBody FinanceReceiptItemVo financeReceiptVo) {
        //receiptId 是必填项，否则会报异常
        receiptItemService.createFinanceReceiptItemWriteOffNo(financeReceiptVo);
        return success(true);
    }

    @PostMapping("/cancelFinanceReceiptItemWriteOffNo")
    @ApiOperation("取消收款明细反核销")
    @Idempotent(timeout = 5)
    public CommonResult<Boolean> cancelFinanceReceiptItemWriteOffNo(@RequestBody FinanceReceiptItemCancelVo
                                                                            financeReceiptVo) {
        receiptItemService.cancelFinanceReceiptItemWriteOffNo(financeReceiptVo);
        return success(true);
    }

    @PostMapping(value = "/import", consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    @ApiOperation("批量银行收款信息导入")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "file", value = "Excel 文件", required = true, dataTypeClass = MultipartFile.class)
    })
    public CommonResult<List<ReceiptItemBatchRespVO>> receiptAccountImport(@RequestParam("file") MultipartFile file) throws IOException {
        List<ReceiptItemBatchCreateReqVO> list = ExcelUtils.read(file, ReceiptItemBatchCreateReqVO.class);
        return success(receiptItemService.receiptItemImport(list));
    }

}
